;    Billarm med en PIC 16F84, ver 1.0
;    This program works with a delay when you open the door. 
;    
;
;	Made by:
;			Andreas Nyholm
;     E-Mail: 	
;			a.nyholm@agrolink.fi
;  	
;
; RA0 = 0 when doors closed, 1 when door/s open
; RA1 = 0 when doors is locked, 1 when not
; RA2 = 0 when ignition is on, 1 when it's off 
; RB0 = LED
; RB1 = Relay 1 min.
; RB2 = Relay 2 min. 

	LIST P=16F84, F=INHX8M, R=DEC 	

#INCLUDE "p16f84.inc"

; Registers 
COUNTER	EQU 12
DLYCNT1	EQU 13
DLYCNT2	EQU 14 

		__CONFIG  _CP_OFF & _WDT_OFF & _XT_OSC
 
; Set up Port A direction register:
     		MOVLW b'00000111'  	; RA0-RA2 inputs
     		TRIS PORTA			
 
; Set up Port B direction register:
    		MOVLW b'00000000'       ; all bits outputs
    		TRIS PORTB
 
; Set Port B output register:
BEGIN   	MOVLW b'00000001'       ; RB0 set high RB1- RB2 low
            MOVWF  PORTB
 
; Wait for ignition off to start program.
WAIT1 	BTFSS PORTA,2         	; RA2 high? (0-> 1 on key)
            GOTO WAIT1            	; not yet
 
; Turn on LED to show activated:
		MOVLW b'00000000'   	; RA2 brought high 
		MOVWF PORTB          	; LED on (RB0)

 
; Wait for doors closed AND doors locked.
WAIT2  	MOVF PORTA, W  		; get port A into W
           	ANDLW b'00000111'  	; check RA0(doors closed), RA1(doors locked) and RA2(ignition off)
            XORLW b'00000100'
		BTFSS STATUS,Z       	; skip when match
           	GOTO WAIT2
 
; Wait for door opened:
; Checks if doors open and make LED blink
WAIT3  	BTFSC PORTA,0		; Skip if RA0 is low
		GOTO TENDELAY		; GOTO 10 sec. countdown if door is open
		MOVLW b'00000001'		; Turn LED off
		MOVWF PORTB			;
		CALL ONETENTH		; 0.5 sec delay
		CALL ONETENTH
		CALL ONETENTH
		CALL ONETENTH	
		CALL ONETENTH
		MOVLW b'00000000'		; Turn LED on
		MOVWF PORTB
		BTFSC PORTA,0		; Skip if RA0 is low		
		GOTO TENDELAY		; GOTO 10 sec. countdown if door is open
		CALL ONETENTH		; 0.3 sec delay
		CALL ONETENTH
		CALL ONETENTH
	  	GOTO WAIT3
 
TENDELAY	MOVLW 100           	; we are going to count 100 * 0.1 = 10 sec
          	MOVWF COUNTER       	; moves 100 to counter
LOOP10    	CALL ONETENTH       	; here is our call to delay .1 sec  
          	BTFSS PORTA,2       	; if RA2 is low, then larm is off 
          	GOTO BEGIN          	; save by the button, no alarm necessary.
          	DECFSZ COUNTER,F       	; finished 10 seconds yet?
          	GOTO LOOP10         	; not yet continue 10 sec countdown.
 			
		  
		MOVLW b'00000110'		; No... Start alarm (RB1 & RB2 high)
            MOVWF PORTB  
		
		CALL TWENTYSEC		; Delay 1 minute
		CALL TWENTYSEC	
		CALL TWENTYSEC
						
            MOVLW b'00000100' 	; RB1 set low
            MOVWF PORTB    	
		
		CALL TWENTYSEC	 	; Delay 1 minute
		CALL TWENTYSEC
            CALL TWENTYSEC
          		
		GOTO BEGIN			

; Subrutines		
ONETENTH  	MOVLW H'0FF'        	; 255 * 385us = 0.982s           
          	MOVWF DLYCNT2
DELAY     	MOVLW H'0FF'        	; 255 decimal
          	MOVWF DLYCNT1     	; 
LOOP1     	DECFSZ DLYCNT1,F   	; reached zero yet? 
          	                 		; loop cnt 385 microsec.
          	GOTO LOOP1          	; not finished inner loop
          	DECFSZ   DLYCNT2,F	; outer loop decrement, finished?
          	GOTO DELAY     		; not finished outer loop.
          	RETURN

TWENTYSEC	MOVLW 200    		; we are going to count 200 * 0.1 = 20 sec
          	MOVWF COUNTER   		 
LOOP20    	CALL ONETENTH     	; here is our call to delay .1 sec
          	DECFSZ COUNTER,F     	; finished 20 seconds yet?     
          	GOTO LOOP20			; not yet continue 20 sec countdown.
		RETURN
		
		END